#pragma once

#include <string>

#include <amdis/Initfile.hpp>
#include <amdis/Output.hpp>
#include <amdis/common/Filesystem.hpp>

namespace AMDiS
{
  // forward declaration
  class AdaptInfo;

  class FileWriterInterface
  {
  public:
    explicit FileWriterInterface(std::string const& base)
    {
      Parameters::get(base + "->filename", filename_);
      Parameters::get(base + "->output directory", dir_);
      Parameters::get(base + "->name", name_);
    }

    virtual ~FileWriterInterface() = default;


    // pure virtual method to be implemented by derived classes
    virtual void writeFiles(AdaptInfo& adaptInfo, bool force) = 0;

  public: // getter and setter

    std::string const& filename() const
    {
      return filename_;
    }

    std::string const& dir() const
    {
      return dir_;
    }

    std::string const& name() const
    {
      return name_;
    }

    void setFilename(std::string const& filename)
    {
      filename_ = filename;
    }

    void setDir(std::string const& dir)
    {
      dir_ = dir;
    }

    void setName(std::string const& name)
    {
      name_ = name;
    }

  protected:
    std::string filename_ = "solution";
    std::string dir_ = ".";
    std::string name_ = "solution";
  };

} // end namespace AMDiS
