#pragma once

#include <amdis/linearalgebra/istl/Communication.hpp>
#include <amdis/linearalgebra/istl/Creators.hpp>
#include <amdis/linearalgebra/Common.hpp>

namespace AMDiS
{
  /** Traits class for a linear solver for the system AX=B using an FE space described by a dune-functions Basis
   *  Contains typedefs specific to the ISTL backend.
   */
  template <class A, class X, class B, class Basis>
  class SolverTraits : public SolverTraitsBase<A,X,B,Basis>
  {
  public:
    using Comm            = ISTLCommunication<Basis>;
    using ScalProd        = Dune::ScalarProduct<X>;
    using LinOp           = Dune::AssembledLinearOperator<A, X, B>;
    using Solver          = Dune::InverseOperator<X, B>;
    using Prec            = Dune::Preconditioner<X, B>;
    using ScalProdCreator = ISTLScalarProductCreator<X>;
    using ParPrecCreator  = ISTLParallelPreconditionerCreator<X, B>;
    using LinOpCreator    = ISTLLinearOperatorCreator<A, X, B>;
  };

} // end namespace AMDiS
